﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;

namespace M51 {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }

        public string[] game = new string[15];
        private string[,] gameBoard = new string[10, 10];
        private Player ally, enemy;
        private string gameId = "0";
        private string url = "http://localhost:8000";
        bool twoPlayers = false;

        private void Form1_Load(object sender, EventArgs e) {
            txtServer.Text = url;
            Size = new Size(Width, 87);
        }

        private void btnNewGame_Click(object sender, EventArgs e) {
            url = txtServer.Text;
            WebRequest webGet = WebRequest.Create(url + "/start");
            LoadGame(webGet);
        }

        private void btnConnect_Click(object sender, EventArgs e) {
            url = txtServer.Text;
            //Nevím jak se připojit k rozehrané mapě bez pohybu
            WebRequest webGet = WebRequest.Create(url + "/tah?id=" + numID.Value + "&pohyb=N");
            LoadGame(webGet);
        }

        private void btnEnd_Click(object sender, EventArgs e) {
            ResetGame();

            MessageBox.Show("Hra byla ukončena hráčem.", "Konec hry", MessageBoxButtons.OK, MessageBoxIcon.Information);
            twoPlayers = false;
            Size = new Size(Width, 87);
            GameControls(false);
            btnNewGame.Focus();
        }

        private void picGame_Paint(object sender, PaintEventArgs e) {
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

            //Players colors
            Brush allyColor = new SolidBrush(Color.FromArgb(0,131,219));
            Brush enemyColor = new SolidBrush(Color.FromArgb(247,110,10));

            //Grid
            for (int i = 0; i < 11; i++) {
                g.DrawLine(Pens.Gray, i * 20, 0, i * 20, 200);
                g.DrawLine(Pens.Gray, 0, i * 20, 200, i * 20);
            }

            //Game
            if (!btnNewGame.Enabled) {
                for (int i = 0; i < 10; i++) {
                    for (int j = 0; j < 10; j++) {
                        //Game Board
                        switch (gameBoard[i, j]) {
                            case "XX": g.FillRectangle(Brushes.Black, i * 20, j * 20, 20, 20); break;
                            case "++": g.DrawImage(Properties.Resources.homeAlly, i * 20, j * 20); break;
                            case "--": g.DrawImage(Properties.Resources.homeEnemy, i * 20, j * 20); break;
                            case "..": break;
                            default:
                                for (int k = 0; k < 99; k++) {
                                    if (k+1 == Convert.ToInt32(gameBoard[i,j]))
                                        g.DrawString((k + 1).ToString(), new Font("Calibri", 10), Brushes.Orange, i * 20 + 2, j * 20 + 2);
                                }
                                break;
                        }
                    }
                }

                //Local player
                g.FillEllipse(allyColor, ally.X * 20 + 5, ally.Y * 20 + 5, 10, 10);
                g.DrawEllipse(Pens.Black, ally.X * 20 + 5, ally.Y * 20 + 5, 10, 10);

                //Server player
                if (twoPlayers) {
                    g.FillEllipse(enemyColor, enemy.X * 20 + 5, enemy.Y * 20 + 5, 10, 10);
                    g.DrawEllipse(Pens.Black, enemy.X * 20 + 5, enemy.Y * 20 + 5, 10, 10);
                }
            }
        }

        private void LoadGame(WebRequest webGet) {
            Stream stream = webGet.GetResponse().GetResponseStream();
            StreamReader sReader = new StreamReader(stream);

            string line = "";
            int x = 0;
            //Saving game to client
            while ((line = sReader.ReadLine()) != null) {
                game[x] = line;
                x++;
            }

            if (game[0].Substring(0, 1) == "C") {
                MessageBox.Show(game[0], "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Saving game board
            for (int i = 0; i < 10; i++) {
                for (int j = 0; j < 10; j++) {
                    gameBoard[j, i] = game[i + 2].Substring(j * 2, 2);
                }
            }
            //Creating players
            ally = new Player();
            if (game[14] != null) {
                twoPlayers = true;
                enemy = new Player();
            }
            //Editing labels
            gameId = game[0];
            lblID.Text = "ID momentální hry: " + gameId;
            lblTahy1.Text = "Počet tahů: " + game[1];
            lblTahy2.Text = lblTahy1.Text;
            lblCelkTahy.Text = "Celkový počet tahů: " + game[1];

            //Editing controls for game
            GameControls(true);

            RefreshPlayers();
            picGame.Refresh();
            Size = new Size(Width, 344);
            picGame.Focus();
        }

        private void ResetGame(){
            for (int i = 0; i < game.Length; i++) {
                game[i] = null;
            }
            for (int i = 0; i < 10; i++) {
                for (int j = 0; j < 10; j++) {
                    gameBoard[i, j] = null;
                }
            }
        }

        private void MoveFunction(string direction) {
            int x = ally.X;
            int y = ally.Y;

            //Invalid moves
            switch (direction) {
                case "N":
                    if (y - 1 < 0 || gameBoard[x, y - 1] == "XX") return;
                    if (twoPlayers && y - 1 == enemy.Y && x == enemy.X) return;
                    if (twoPlayers && gameBoard[x, y - 1] == "--") return;
                    break;
                case "P":
                    if (x + 1 > 9 || gameBoard[x + 1, y] == "XX") return;
                    if (twoPlayers && x + 1 == enemy.X && y == enemy.Y) return;
                    if (twoPlayers && gameBoard[x + 1, y] == "--") return;
                    break;
                case "D":
                    if (y + 1 > 9 || gameBoard[x, y + 1] == "XX") return;
                    if (twoPlayers && y + 1 == enemy.Y && x == enemy.X) return;
                    if (twoPlayers && gameBoard[x, y + 1] == "--") return;
                    break;
                case "L":
                    if (x - 1 < 0 || gameBoard[x - 1, y] == "XX") return;
                    if (twoPlayers && x - 1 == enemy.X && y == enemy.Y) return;
                    if (twoPlayers && gameBoard[x - 1, y] == "--") return;
                    break;
            }

            WebRequest webGet = WebRequest.Create(url+"/tah?id=" + game[0] + "&pohyb=" + direction);
            Stream stream = webGet.GetResponse().GetResponseStream();
            StreamReader sReader = new StreamReader(stream);

            string line = "";
            x = 0;
            //Refreshing game
            while ((line = sReader.ReadLine()) != null) {
                game[x] = line;
                x++;
            }
            x = 0;
            //Refreshing game board
            for (int i = 0; i < 10; i++) {
                for (int j = 0; j < 10; j++) {
                    gameBoard[j, i] = game[i + 2].Substring(j * 2, 2);
                }
            }

            //Game over
            if (game[0].Substring(0, 1) == "K") {
                string[] end = game[0].Substring("Konec hry: ".Length).Split(' ');
                if (twoPlayers)
                    MessageBox.Show("Konec hry!\n\nPočet bodů:\nHráč 1: " + end[0] + "\nHráč 2: " + end[1], "Konec hry č." + gameId, MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("Konec hry!\n\nPočet bodů:\nHráč 1: " + end[0], "Konec hry č." + gameId, MessageBoxButtons.OK, MessageBoxIcon.Information);

                ResetGame();
                twoPlayers = false;
                Size = new Size(Width, 87);
                GameControls(false);
                btnNewGame.Focus();
                return;
            }
            //Error
            else if (game[0].Substring(0, 1) == "C") {
                MessageBox.Show(game[0], "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            lblTahy1.Text = "Počet tahů: " + game[1];
            lblTahy2.Text = lblTahy1.Text;
            RefreshPlayers();
            picGame.Refresh();
        }

        private void GameControls(bool started) {
            btnEnd.Enabled = started;
            btnNewGame.Enabled = !started;
            btnConnect.Enabled = !started;
            numID.Enabled = !started;
            txtServer.Enabled = !started;
            lblCelkTahy.Visible = started;
            lblID.Visible = started;

            gbPlayer1.Visible = started;
            if (twoPlayers)
                gbPlayer2.Visible = started;
        }

        private void RefreshPlayers() {
            string[] stats = game[12].Split(' ');

            ally.RefreshPlayer(stats);
            lblTreasure1.Text = "Poklad: " + ally.Treasure;
            lblPoints1.Text = "Body: " + ally.Points;

            if (twoPlayers) {
                stats = game[13].Split(' ');
                enemy.RefreshPlayer(stats);
                lblTreasure2.Text = "Poklad: " + enemy.Treasure;
                lblPoints2.Text = "Body: " + enemy.Points;
            }
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e) {
            switch (e.KeyCode) { 
                case Keys.W: MoveFunction("N"); break;
                case Keys.D: MoveFunction("P"); break;
                case Keys.S: MoveFunction("D"); break;
                case Keys.A: MoveFunction("L"); break;
            }
        }
    }
}
